-- This code is dedicated to the public domain. 2017, Moritz Neikes


-- the url that points to the file
local url = "http://imgsrc.hubblesite.org/hvi/uploads/image_file/image_attachment/30217/STSCI-H-p1729a-z-1000x800.png"
-- the name of the file
local filename = "out.png"

local image = nil

-- a table that contains the current download status
local dl_status = nil

-- two channels to communicate with the downloader thread
local requests_channel = love.thread.getChannel("requests")
local progress_channel = love.thread.getChannel("progress")

function love.load()
  love.filesystem.setIdentity("download_progress")

  -- start the downloader thread
  local downloader_thread = love.thread.newThread("downloader.lua")
  downloader_thread:start()
end

function love.draw()
  love.graphics.setColor(255, 255, 255)
  love.graphics.print("Press (space) to start the download", 0, 0)
  love.graphics.translate(0, 16)
  love.graphics.rectangle("line", 10, 10, 104, 14)
  if dl_status then
    if dl_status.err then
      love.graphics.setColor(255, 0, 0)
    elseif dl_status.finished then
      love.graphics.setColor(0, 255, 0)
    end
    if dl_status.file_size > 0 then
      local perc =  dl_status.downloaded / dl_status.file_size
      perc = math.floor(perc * 100)
      love.graphics.rectangle("fill", 12, 12, perc, 10)
      love.graphics.print(string.format("%d%%", perc), 116, 10)
    end
    if dl_status.err then
      love.graphics.print(dl_status.err, 0, 32)
    end
  end
  if image then
    love.graphics.setColor(255, 255, 255)
    love.graphics.draw(image, 0, 0, 0, .25, .25)
  end
end

function love.update()
  -- Read all progress updates that came in since the last frame.
  -- If there is no update, the returned value will be nil
  local progress_update = progress_channel:pop()
  while progress_update do

    -- process the progress update
    if progress_update.finished then
      dl_status.finished = true
      -- Load the downloaded image
      image = love.graphics.newImage(filename)
    end
    if progress_update.file_size then
      dl_status.file_size = progress_update.file_size
    end
    if progress_update.chunk then
      dl_status.downloaded = dl_status.downloaded + progress_update.chunk
    end
    if progress_update.error then
      dl_status.err = progress_update.error
    end

    -- get the next progress update
    -- If there is no update, the returned value will be nil
    progress_update = progress_channel:pop()
  end
end

local function start_download(url, filename)

  -- remove the previously loaded image
  image = nil

  -- reset the download status
  dl_status = {
    finished = false, -- whether the download has finished
    downloaded = 0,   -- the total number of bytes that were downloaded so far
    file_size = 0,    -- the size of the file
  }

  -- construct the request table that will be sent to the download thread
  local request = {
    url = url,
    filename = filename
  }

  -- send the request to the downloader thread via the requests channel
  requests_channel:push(request)

end

function love.keypressed(key)
  if key == "space" then
    start_download(url, filename)
  end
end

-- if the downloader thread throws an error, this makes sure that we actually
-- see that error
function love.threaderror(thread, err)
  error(err)
end